library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity VGA_mux is
port(
    clk        : in  std_logic;
    reset      : in  std_logic;
    read       : in  std_logic;
    write      : in  std_logic;
    chipselect : in  std_logic;
    address    : in  std_logic_vector(18 downto 0);
    readdata   : out std_logic_vector(15 downto 0);
    writedata  : in  std_logic_vector(15 downto 0);
	 byteenable : in  std_logic_vector(1 downto 0);
	 
	 
	 VGA_CLK,                         -- Clock
    VGA_HS,                          -- H_SYNC
    VGA_VS,                          -- V_SYNC
    VGA_BLANK,                       -- BLANK
    VGA_SYNC : out std_logic;        -- SYNC
    VGA_R,                           -- Red[9:0]
    VGA_G,                           -- Green[9:0]
    VGA_B : out std_logic_vector(9 downto 0); -- Blue[9:0]
	 
	 SRAM_DQ   : inout std_logic_vector(15 downto 0);
    SRAM_ADDR : out std_logic_vector(18 downto 0);
    SRAM_UB_N, SRAM_LB_N : out std_logic;
    SRAM_WE_N, SRAM_CE_N : out std_logic;
    SRAM_OE_N            : out std_logic
	 ); 
end VGA_mux;


architecture rtl of VGA_mux is

component de2_sram_controller is 
  port (
      chipselect : in std_logic;
      write, read : in std_logic;
      address  :  in std_logic_vector(18 downto 0);
      readdata : out std_logic_vector(15 downto 0);
      writedata : in std_logic_vector(15 downto 0);
      byteenable : in std_logic_vector(1 downto 0);

      SRAM_DQ : inout std_logic_vector(15 downto 0);
      SRAM_ADDR : out std_logic_vector(18 downto 0);
      SRAM_UB_N, SRAM_LB_N : out std_logic;
      SRAM_WE_N, SRAM_CE_N : out std_logic;
      SRAM_OE_N            : out std_logic
    );
	 end component de2_sram_controller;
	 
component de2_vga_raster is
  port (
    clk   : in std_logic;                    -- Should be 25.125 MHz
    reset : in std_logic;

    read       : in  std_logic;
    write      : in  std_logic;
    chipselect : in  std_logic;
    address    : in  std_logic_vector(18 downto 0);
    readdata   : out std_logic_vector(18 downto 0);
    writedata  : in  std_logic_vector(15 downto 0);
	 write_inf  : in  std_logic_vector(15 downto 0);
	 address_inf: in  std_logic_vector(18 downto 0);

    VGA_CLK,                         -- Clock
    VGA_HS,                          -- H_SYNC
    VGA_VS,                          -- V_SYNC
    VGA_BLANK,                       -- BLANK
    VGA_SYNC : out std_logic;        -- SYNC
    VGA_R,                           -- Red[9:0]
    VGA_G,                           -- Green[9:0]
    VGA_B : out std_logic_vector(9 downto 0) -- Blue[9:0]
	 
    );
	 end component de2_vga_raster;
	 
	 signal internal_VGA_R :  STD_LOGIC_VECTOR (9 DOWNTO 0);
    signal internal_VGA_G :  STD_LOGIC_VECTOR (9 DOWNTO 0);
    signal internal_VGA_B :  STD_LOGIC_VECTOR (9 DOWNTO 0);
    signal internal_VGA_SYNC :   STD_LOGIC;
    signal internal_VGA_BLANK :  STD_LOGIC;
    signal internal_VGA_VS :     STD_LOGIC ;
	 signal internal_VGA_HS : STD_LOGIC;
	 signal internal_VGA_CLK : STD_LOGIC;
	 signal internal_readdata: std_logic_vector(15 downto 0);
	 signal address_from_VGA: std_logic_vector(18 downto 0);
	 signal internal_readdata_sram_vga : std_logic_vector(15 downto 0);
	 signal data_to_VGA : std_logic_vector(15 downto 0);
    signal data_to_SRAM: std_logic_vector(15 downto 0);
    signal read_address: std_logic_vector(18 downto 0);
    signal read_enable : std_logic;
	 signal read_abc:std_logic;
	 signal write_abc:std_logic;
	 signal address_abc:std_logic_vector(18 downto 0);
	 signal chipselect_vga :std_logic;
	 signal read_sram      : std_logic;
	 signal flag           :std_logic:='0';
	 signal write_0        :std_logic;
	 signal counter        :unsigned(3 downto 0);
	 signal counter_address        :unsigned(15 downto 0);
	 signal clk25 : std_logic := '0';
	 signal clk_half: std_logic :='0';
	 signal sram_write     :std_logic;
	 signal sram_read      :std_logic;
	 signal sram_address   :std_logic_vector(18 downto 0);
	 signal sram_writedata :std_logic_vector(15 downto 0);
	 signal chipselect_sram:std_logic;
	 signal chipselect_abc :std_logic;
	 signal write_inf      :std_logic_vector(15 downto 0):=(others => '0');
	 signal address_inf    :std_logic_vector(18 downto 0);
	 signal mode_control   :std_logic_vector(18 downto 0);
	 
	 
	 begin 

    V1:  de2_vga_raster port map (
    clk => clk,
	 reset => reset,
	 address => address_abc,
	 writedata => internal_readdata,
	 read => read_abc,
	 write => write_abc,
	 chipselect => chipselect_vga,
	 readdata  => address_from_VGA,
	 write_inf => write_inf,
	 address_inf => address_inf,
	 
	 VGA_CLK => internal_VGA_CLK,
	 VGA_HS => internal_VGA_HS,
	 VGA_VS => internal_VGA_VS,
	 VGA_BLANK => internal_VGA_BLANK,
	 VGA_SYNC => internal_VGA_SYNC,
	 VGA_R => internal_VGA_R,
	 VGA_G => internal_VGA_G,
	 VGA_B => internal_VGA_B 
  );
  
  V_final:de2_sram_controller port map(
	 chipselect => chipselect_sram,
	 write => sram_write,
	 read =>  sram_read,
	 address => sram_address,
	 readdata => internal_readdata_sram_vga,
	 writedata => sram_writedata,
	 byteenable => byteenable,
	 
	 SRAM_DQ   => SRAM_DQ,
	 SRAM_ADDR => SRAM_ADDR,
	 SRAM_UB_N => SRAM_UB_N,
	 SRAM_LB_N => SRAM_LB_N,
	 SRAM_WE_N => SRAM_WE_N,
	 SRAM_CE_N => SRAM_CE_N,
	 SRAM_OE_N => SRAM_OE_N
	 );
	 
	 flag_process: process(clk)
	 begin
	     if rising_edge(clk) then
	       mode_control<=address;
			 if chipselect = '1' and write = '1' then
			  if mode_control = "1111100100000110000" then
			      flag <= '1';
			  elsif mode_control = "1111100100000101000" then
			      flag <= '0';
			  end if;
		end if;
	 end if;
	 end process;
	 
	 V_f:  process(clk)
	 begin
	    if rising_edge(clk) then
		     if chipselect = '1' and write = '1' then
			  
			    if  flag = '0' then
		         chipselect_vga <= '0';
					chipselect_sram<= '1';
		         write_abc <= '0';
					sram_write<= '1';
					read_abc  <= '0';
					sram_read <= '0';
					sram_writedata <= writedata;
		         address_abc <= "0000000000000000000";
					sram_address <= address;
					
				 else	
					address_inf    <= address;
			      write_inf      <= writedata;
					chipselect_vga <= '1';
					chipselect_sram<='1';
					write_abc <= '1';
					read_abc <= '0';
					sram_read<='1';
					read_abc <= '0';
				   address_abc <= "0000000000000000000";	
					sram_address <= address_from_VGA;
					internal_readdata<=internal_readdata_sram_vga;
				 end if;	
					
           else
			     
			      chipselect_vga <= '1';
					chipselect_sram<='1';
					write_abc <= '1';
					sram_write <= '0';
					read_abc <= '0';
					sram_read<='1';	
	            address_abc <= "0000000000000000000";
					sram_address <= address_from_VGA;-----signal????
					internal_readdata<=internal_readdata_sram_vga;
					end if;
            end if;
	 end process;
  

--	V2:de2_sram_controller port map(
--	 chipselect => chipselect_abc,
--	 write => '0',
--	 read =>  write_abc,
--	 address => read_address,
--	 readdata => internal_readdata_sram_vga,
--	 writedata => writedata,
--	 byteenable => byteenable,
--	 
--	 SRAM_DQ   => SRAM_DQ,
--	 SRAM_ADDR => SRAM_ADDR,
--	 SRAM_UB_N => SRAM_UB_N,
--	 SRAM_LB_N => SRAM_LB_N,
--	 SRAM_WE_N => SRAM_WE_N,
--	 SRAM_CE_N => SRAM_CE_N,
--	 SRAM_OE_N => SRAM_OE_N
--	 );
--	V2:de2_sram_controller port map(
--	 chipselect => chipselect,
--	 write => write,
--	 read =>  read,
--	 address => address,
--	 readdata => readdata,
--	 writedata => writedata,
--	 byteenable => byteenable,
--	 
--	 SRAM_DQ   => SRAM_DQ,
--	 SRAM_ADDR => SRAM_ADDR,
--	 SRAM_UB_N => SRAM_UB_N,
--	 SRAM_LB_N => SRAM_LB_N,
--	 SRAM_WE_N => SRAM_WE_N,
--	 SRAM_CE_N => SRAM_CE_N,
--	 SRAM_OE_N => SRAM_OE_N
--	 );
--  
  VGA_R<=internal_VGA_R;
  VGA_G<=internal_VGA_G;
  VGA_B<=internal_VGA_B;
  VGA_SYNC<=internal_VGA_SYNC;
  VGA_BLANK<=internal_VGA_BLANK;
  VGA_VS<=internal_VGA_VS;
  VGA_HS<=internal_VGA_HS;
  VGA_CLK<=internal_VGA_CLK;
--
 

  
--  process(clk)
--	 begin
--	    if rising_edge(clk) then
--		    -- if write_enable = '1' then
--		         chipselect_vga <= '1';
--		         write_abc <= '1';
--		         internal_readdata <= internal_readdata_sram_vga;
--		         address_abc <= "0000000000000000000";
--		         read_sram <= '1';
--					read_address <= address_from_VGA;
--           --elsif write_enable = '0' then
--					
--            end if;
--	 end process;


end architecture;